#!/usr/bin/env python3


import json
import redis

from ..util import merge, REQUIRED
from .driver import Driver


class RedisDriver(Driver):
    client: redis.Redis

    def __init__(self, args):
        args = merge(args, {
            "host": 'localhost',
            "port": 6379,
            "db": 0,
            "password": None,
        })
        self.client = redis.Redis(
            host=args["host"],
            port=args["port"],
            db=args["db"],
            password=args["password"],
            decode_responses=True,
        )

    def name(self, req):
        return req["cmd"]

    def do(self, req):
        req = merge(req, {
            "cmd": REQUIRED,
        })

        do_map = {
            "set": self.set,
            "get": self.get,
            "setJson": self.set_json,
            "getJson": self.get_json,
            "hset": self.hset,
            "hget": self.hget,
            "del": self.delete,
            "hdel": self.hdel,
        }

        if req["cmd"] not in do_map:
            raise Exception("unsupported cmd [{}]".format(req["cmd"]))

        return do_map[req["cmd"]](req)

    def set(self, req):
        req = merge(req, {
            "key": REQUIRED,
            "val": REQUIRED,
            "exp": None,
        })
        ok = self.client.set(req["key"], req["val"], ex=req["exp"])
        return {"ok": ok}

    def get(self, req):
        req = merge(req, {
            "key": REQUIRED
        })
        val = self.client.get(req["key"])
        return {"val": val}

    def delete(self, req):
        req = merge(req, {
            "key": REQUIRED,
        })
        ok = self.client.delete(req["key"])
        return {"ok": ok}

    def set_json(self, req):
        req = merge(req, {
            "key": REQUIRED,
            "val": REQUIRED,
            "exp": None,
        })
        ok = self.client.set(req["key"], json.dumps(req["val"]), ex=req["exp"])
        return {"ok": ok}

    def get_json(self, req):
        req = merge(req, {
            "key": REQUIRED
        })
        res = self.client.get(req["key"])
        return json.loads(res)

    def hset(self, req):
        req = merge(req, {
            "key": REQUIRED,
            "field": REQUIRED,
            "val": REQUIRED,
        })
        n = self.client.hset(req["key"], req["field"], req["val"])
        return {"n": n}

    def hget(self, req):
        req = merge(req, {
            "key": REQUIRED,
            "field": REQUIRED,
        })
        val = self.client.hget(req["key"], req["field"])
        return {"val": val}

    def hdel(self, req):
        req = merge(req, {
            "key": REQUIRED,
            "field": REQUIRED,
        })
        n = self.client.hdel(req["key"], req["field"])
        return {"n": n}



#!/usr/bin/env python3


import pymysql

from ..util import merge, REQUIRED
from .driver import Driver


class MysqlDriver(Driver):
    connection: pymysql.Connection

    def __init__(self, args):
        args = merge(args, {
            "host": "localhost",
            "port": 3306,
            "username": "root",
            "password": REQUIRED,
            "database": REQUIRED,
        })

        self.connection = pymysql.connect(
            host=args["host"],
            port=args["port"],
            user=args["username"],
            password=args["password"],
            database=args["database"],
            cursorclass=pymysql.cursors.DictCursor,
        )

    def name(self, req):
        return req["sql"].split(" ")[0]

    def do(self, req):
        req = merge(req, {
            "cmd": "sql",
            "sql": REQUIRED,
            "args": [],
        })

        do_map = {
            "sql": self.sql,
            "fetchone": self.fetchone,
            "fetchall": self.fetchall,
        }
        if req["cmd"] not in do_map:
            raise Exception("unsupported cmd [{}]".format(req["cmd"]))

        try:
            return do_map[req["cmd"]](req)
        except pymysql.err.OperationalError as e:
            return {
                "code": "OperationalError",
                "err": {
                    "type": "OperationalError",
                    "args": list(e.args)
                },
            }
        except Exception as e:
            raise e

    def sql(self, req):
        with self.connection.cursor() as cursor:
            cursor.execute(req["sql"], req["args"])
        self.connection.commit()
        return {
            "code": "OK"
        }

    def fetchone(self, req):
        with self.connection.cursor() as cursor:
            cursor.execute(req["sql"], req["args"])
            result = cursor.fetchone()
        self.connection.commit()
        return {
            "code": "OK",
            "res": result
        }

    def fetchall(self, req):
        with self.connection.cursor() as cursor:
            cursor.execute(req["sql"], req["args"])
            result = cursor.fetchall()
        self.connection.commit()
        return {
            "code": "OK",
            "res": result
        }



#!/usr/bin/env python3


import json

from ..result import TestResult
from ..util import merge
from .reporter import Reporter


class JsonReporter(Reporter):
    def __init__(self, args=None):
        super().__init__(args)
        args = merge(args, {
            "indent": None
        })
        self.indent = args["indent"]

    def report(self, res: TestResult) -> str:
        return json.dumps(res, indent=self.indent, default=lambda x: x.to_json())



#!/usr/bin/env python3


import json
import mns.mns_client
import mns.mns_exception
from ..util import merge, REQUIRED
from .driver import Driver


class MNSDriver(Driver):
    client: mns.mns_client.MNSClient

    def __init__(self, args):
        args = merge(args, {
            "AccessKeyId": REQUIRED,
            "AccessKeySecret": REQUIRED,
            "Endpoint": REQUIRED,
        })
        self.client = mns.mns_client.MNSClient(
            host=args["Endpoint"],
            access_id=args["AccessKeyId"],
            access_key=args["AccessKeySecret"],
        )

    def name(self, req):
        return req["Action"]

    def do(self, req):
        req = merge(req, {
            "Action": REQUIRED
        })

        do_map = {
            "PublishMessage": self.publish_message,
            "ReceiveMessage": self.receive_message,
            "DeleteMessage": self.delete_message,
        }
        if req["Action"] not in do_map:
            raise Exception("unsupported action [{}]".format(req["Action"]))

        try:
            return do_map[req["Action"]](req)
        except mns.mns_exception.MNSServerException as e:
            return {
                "Type": e.type,
                "Message": e.message,
                "HostId": e.host_id,
                "RequestId": e.request_id,
            }
        except Exception as e:
            raise e

    def publish_message(self, req):
        req = merge(req, {
            "Topic": REQUIRED,
            "MessageBody": "",
            "Body": {},
            "MessageTag": "",
            "DirectMail": None,
            "DirectSms": None,
        })

        req = mns.mns_client.PublishMessageRequest(
            topic_name=req["Topic"],
            message_body=req["MessageBody"] if req["MessageBody"] else json.dumps(req["Body"]),
            message_tag=req["MessageTag"],
            direct_mail=req["DirectMail"],
            direct_sms=req["DirectSms"],
        )
        res = mns.mns_client.PublishMessageResponse()
        self.client.publish_message(req, res)

        return {
            "Status": res.status,
            "Header": res.header,
            "RequestId": res.get_requestid(),
            "MessageId": res.message_id,
            "MessageBodyMD5": res.message_body_md5,
        }

    def receive_message(self, req):
        req = merge(req, {
            "Queue": REQUIRED,
            "Base64Decode": False,
            "WaitSeconds": -1,
            "AutoRelease": True,
        })

        receive_message_req = mns.mns_client.ReceiveMessageRequest(
            queue_name=req["Queue"],
            base64decode=req["Base64Decode"],
            wait_seconds=req["WaitSeconds"],
        )
        receive_message_res = mns.mns_client.ReceiveMessageResponse()

        self.client.receive_message(receive_message_req, receive_message_res)

        body = None
        try:
            body = json.loads(receive_message_res.message_body)
            body["Body"] = json.loads(body["Message"])
        except json.decoder.JSONDecodeError as e:
            pass
        except Exception as e:
            raise e

        if req["AutoRelease"]:
            delete_message_req = mns.mns_client.DeleteMessageRequest(
                queue_name=req["Queue"],
                receipt_handle=receive_message_res.receipt_handle,
            )
            delete_message_res = mns.mns_client.DeleteMessageResponse()
            self.client.delete_message(delete_message_req, delete_message_res)

        return {
            "Status": receive_message_res.status,
            "Header": receive_message_res.header,
            "RequestId": receive_message_res.get_requestid(),
            "MessageId": receive_message_res.message_id,
            "ReceiptHandle": receive_message_res.receipt_handle,
            "MessageBodyMD5": receive_message_res.message_body_md5,
            "MessageBody": receive_message_res.message_body,
            "EnqueueTime": receive_message_res.enqueue_time,
            "NextVisibleTime": receive_message_res.next_visible_time,
            "FirstDequeueTime": receive_message_res.first_dequeue_time,
            "DequeueCount": receive_message_res.dequeue_count,
            "Priority": receive_message_res.priority,
            "Body": body,
        }

    def delete_message(self, req):
        req = merge(req, {
            "Queue": REQUIRED,
            "ReceiptHandle": REQUIRED,
        })

        req = mns.mns_client.DeleteMessageRequest(
            queue_name=req["Queue"],
            receipt_handle=req["ReceiptHandle"]
        )
        res = mns.mns_client.DeleteMessageResponse()

        try:
            self.client.delete_message(req, res)
        except Exception as e:
            raise e

        return {
            "Status": res.status,
            "Header": res.header,
            "RequestId": res.get_requestid(),
        }



#!/usr/bin/env python3


import json
import locale
from types import SimpleNamespace

from ..util import merge


i18n = {
    "dft": {
        "title": {
            "report": "REPORT",
            "test": "TEST",
        },
        "status": {
            "succ": "PASS",
            "skip": "SKIP",
            "fail": "FAIL",
        },
        "summary": {
            "caseTotal": "Total Case",
            "casePass": "Case Pass",
            "caseSkip": "Case Skip",
            "caseFail": "Case Fail",
            "stepPass": "Step Pass",
            "stepSkip": "Step Skip",
            "stepFail": "Step Fail",
            "assertionPass": "Assertion Pass",
            "assertionFail": "Assertion Fail",
            "elapse": "Elapse",
        },
        "testHeader": {
            "description": "Description",
            "err": "Err",
            "setUp": "SetUp",
            "case": "Case",
            "tearDown": "TearDown",
            "subTest": "SubTest",
        },
        "caseHeader": {
            "description": "Description",
            "command": "Command",
            "beforeCaseStep": "BeforeCaseStep",
            "preStep": "PreStep",
            "step": "Step",
            "postStep": "PostStep",
            "afterCaseStep": "AfterCaseStep",
        },
        "stepHeader": {
            "description": "Description",
            "req": "Req",
            "res": "Res",
            "err": "Err",
            "subStep": "SubStep",
        },
        "toolTips": {
            "copy": "copy"
        }
    },
    "zh": {
        "title": {
            "report": "测试报告",
            "test": "测试",
        },
        "status": {
            "succ": "通过",
            "skip": "跳过",
            "fail": "失败",
        },
        "summary": {
            "caseTotal": "测试总数",
            "casePass": "测试通过",
            "caseSkip": "测试跳过",
            "caseFail": "测试失败",
            "stepPass": "步骤通过",
            "stepSkip": "步骤跳过",
            "stepFail": "步骤失败",
            "assertionPass": "断言通过",
            "assertionFail": "断言失败",
            "elapse": "耗时",
        },
        "testHeader": {
            "description": "描述",
            "err": "错误",
            "setUp": "测试环境准备",
            "case": "测试单元",
            "tearDown": "测试环境清理",
            "subTest": "子测试",
        },
        "caseHeader": {
            "description": "描述",
            "command": "命令",
            "beforeCaseStep": "测试开始前",
            "preStep": "前置步骤",
            "step": "步骤",
            "postStep": "后置步骤",
            "afterCaseStep": "测试结束后",
        },
        "stepHeader": {
            "description": "描述",
            "req": "请求",
            "res": "返回",
            "err": "错误",
            "subStep": "子步骤",
        },
        "toolTips": {
            "copy": "复制"
        }
    }
}


class I18n:
    def __init__(self, args=None):
        lang = "dft"
        try:
            lang = locale.getdefaultlocale()[0].split('_')[0]
        except Exception as e:
            pass

        if args is None:
            args = {}
        args = merge(args, {
            "lang": lang,
            "i18n": {},
        })

        lang = args["lang"]
        if lang not in i18n:
            lang = "dft"

        self.i18n_ = json.loads(
            json.dumps(merge(args["i18n"], merge(i18n[lang], i18n["dft"]))),
            object_hook=lambda x: SimpleNamespace(**x),
        )

    def i18n(self):
        return self.i18n_



#!/usr/bin/env python3


from .reporter import Reporter
from .json_reporter import JsonReporter
from .text_reporter import TextReporter
from .html_reporter import HtmlReporter
from .format_step_res import format_step_res


reporter_map = {
    "text": TextReporter,
    "json": JsonReporter,
    "html": HtmlReporter,
    "none": Reporter,
}


__all__ = [
    "Reporter",
    "JsonReporter",
    "TextReporter",
    "HtmlReporter",
    "reporter_map",
    "format_step_res",
]



#!/usr/bin/env python3


import unittest
import json
from .render import render


class TestRender(unittest.TestCase):
    def test_render(self):
        res = render({
            "key1": "val1",
            "#key2": "case['key2']",
            "key3": [{
                "key4": "val4",
                "#key5": "case['key5']"
            }],
            "$key6": "echo -n 'val6'",
        }, {
            "key2": 2,
            "key5": "val5"
        })
        print(json.dumps(res, indent=True))
        self.assertDictEqual(res, {
            "key1": "val1",
            "key2": 2,
            "key3": [{
                "key4": "val4",
                "key5": "val5"
            }],
            "key6": "val6"
        })


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python3


from .hook import Hook
from .debug_hook import DebugHook
from .report_hook import ReportHook
from .email_hook import EmailHook
from .stdlog_hook import StdLogHook
from .slslog_hook import SLSLogHook


hook_map = {
    "debug": DebugHook,
    "report": ReportHook,
    "email": EmailHook,
    "stdLog": StdLogHook,
    "slsLog": SLSLogHook,
}

__all__ = [
    "Hook",
    "DebugHook",
    "EmailHook",
    "ReportHook",
    "StdLogHook",
    "SLSLogHook",
    "hook_map",
]



#!/usr/bin/env python3


from setuptools import setup, find_packages


with open("README.rst") as fp:
    readme = fp.read()


with open("LICENSE") as fp:
    license_ = fp.read()


setup(
    name="qas",
    version="1.0.3",
    description="A general, concurrent and extensible functional testing framework",
    long_description=readme,
    author="hatlonely",
    author_email="hatlonely@foxmail.com",
    url="https://github.com/hatlonely/qas",
    license=license_,
    packages=find_packages(exclude=("tests", "docs", "tmp", "ops", "build", "dist")),
    scripts=["bin/qas"],
    keywords=["qas"],
    install_requires=[
        "requests~=2.27.1",
        "PyYAML~=6.0",
        "colorama~=0.4.4",
        "tablestore~=5.1.0",
        "python-dateutil~=2.8.1",
        "aliyun_python_sdk_core==2.13.30",
        "aliyunsdkcore~=1.0.3",
        "PyMySQL~=1.0.2",
        "redis~=4.1.0",
        "setuptools~=57.4.0",
        "durationpy~=0.5",
        "aliyun-mns-sdk~=1.1.6",
        "oss2~=2.13.0",
        "pymongo~=4.0.1",
        "pycrypto~=2.6.1",
        "Jinja2~=3.0.3",
        "Markdown~=3.3.6",
        "grpcio~=1.43.0",
        "grpcio-tools~=1.43.0",
        "protobuf~=3.19.3",
        "thrift~=0.15.0",
        "elasticsearch~=7.16.0",
        "aliyun-log-python-sdk~=0.7.5"
    ],
)



#!/usr/bin/env python3


import json
import unittest
import oss2

from .oss_driver import OSSDriver


access_key_id = "xx"
access_key_secret = "xx"
endpoint = "http://oss-cn-shanghai.aliyuncs.com"
bucket = "xx"


class TestOSS(unittest.TestCase):
    def setUp(self) -> None:
        self.client = oss2.Bucket(
            oss2.Auth(
                access_key_id=access_key_id,
                access_key_secret=access_key_secret,
            ),
            endpoint=endpoint,
            bucket_name=bucket,
        )

    def test_get_object_meta(self):
        res = self.client.get_object_meta(key="test-file.txt")
        res = {
            "Status": res.status,
            "Headers": dict([(i, res.headers[i]) for i in res.headers]),
            "RequestId": res.request_id,
            "Etag": res.etag,
            "ContentLength": res.content_length,
            "LastModified": res.last_modified,
            "VersionId": res.versionid,
            "DeleteMarker": res.delete_marker,
        }
        print(json.dumps(res, indent=True))

    def test_sign_url(self):
        res = self.client.sign_url(
            method="GET",
            key="test-file.txt",
            expires=1800,
            headers=None,
            params=None,
        )
        print(res)

    def test_get_object_to_file(self):
        res = self.client.get_object_to_file("test-file.txt", "test-file.txt")
        res = {
            "Status": res.status,
            "Headers": dict([(i, res.headers[i]) for i in res.headers]),
            "RequestId": res.request_id,
            "Etag": res.etag,
            "ContentLength": res.content_length,
            "LastModified": res.last_modified,
            "VersionId": res.versionid,
            "DeleteMarker": res.delete_marker,
        }
        print(json.dumps(res, indent=True))

    def test_put_object_from_file(self):
        res = self.client.put_object_from_file("test-file.txt", "test-file.txt")
        res = {
            "Status": res.status,
            "Headers": dict([(i, res.headers[i]) for i in res.headers]),
            "RequestId": res.request_id,
            "Etag": res.etag,
            "VersionId": res.versionid,
            "DeleteMarker": res.delete_marker,
        }
        print(json.dumps(res, indent=True))


class TestOSSDriver(unittest.TestCase):
    def setUp(self) -> None:
        self.driver = OSSDriver({
            "AccessKeyId": access_key_id,
            "AccessKeySecret": access_key_secret,
            "Endpoint": endpoint,
            "Bucket": bucket,
        })

    def test_get_object_meta(self):
        res = self.driver.do({
            "Action": "GetObjectMeta",
            "Key": "test-file.txt"
        })
        print(json.dumps(res, indent=True))

    def test_sign_url(self):
        res = self.driver.do({
            "Action": "SignURL",
            "Key": "test-file.txt"
        })
        print(json.dumps(res))

    def test_get_object_to_file(self):
        res = self.driver.do({
            "Action": "GetObjectToFile",
            "Key": "test-file.txt",
            "Filename": "test-file.txt"
        })
        print(json.dumps(res, indent=True))

    def test_put_object_from_file(self):
        res = self.driver.do({
            "Action": "PutObjectFromFile",
            "Key": "test-file.txt",
            "Filename": "test-file.txt"
        })
        print(json.dumps(res, indent=True))


if __name__ == "__main__":
    pass



#!/usr/bin/env python3


import oss2

from ..util import merge, REQUIRED
from .driver import Driver


class OSSDriver(Driver):
    client: oss2.Bucket

    def __init__(self, args):
        args = merge(args, {
            "AccessKeyId": REQUIRED,
            "AccessKeySecret": REQUIRED,
            "Endpoint": REQUIRED,
            "Bucket": REQUIRED,
        })

        self.client = oss2.Bucket(
            oss2.Auth(
                access_key_id=args["AccessKeyId"],
                access_key_secret=args["AccessKeySecret"],
            ),
            endpoint=args["Endpoint"],
            bucket_name=args["Bucket"],
        )

    def name(self, req):
        return req["Action"]

    def do(self, req):
        req = merge(req, {
            "Action": REQUIRED
        })

        do_map = {
            "GetObjectMeta": self.get_object_meta,
            "GetObjectToFile": self.get_object_to_file,
            "PutObjectFromFile": self.put_object_from_file,
            "SignURL": self.sign_url,
        }
        if req["Action"] not in do_map:
            raise Exception("unsupported action [{}]".format(req["Action"]))

        try:
            return do_map[req["Action"]](req)
        except oss2.exceptions.NoSuchKey as e:
            return {
                "Status": e.status,
                "RequestId": e.request_id,
                "Code": e.code,
                "Message": e.message,
                "Details": e.details,
            }
        except Exception as e:
            raise e

    def get_object_meta(self, req):
        req = merge(req, {
            "Key": REQUIRED,
            "Headers": None,
            "Params": None,
        })
        res = self.client.get_object_meta(key=req["Key"], params=req["Params"], headers=req["Headers"])
        return {
            "Status": res.status,
            "Headers": dict([(i, res.headers[i]) for i in res.headers]),
            "RequestId": res.request_id,
            "Etag": res.etag,
            "ContentLength": res.content_length,
            "LastModified": res.last_modified,
            "VersionId": res.versionid,
            "DeleteMarker": res.delete_marker,
        }

    def get_object_to_file(self, req):
        req = merge(req, {
            "Key": REQUIRED,
            "Filename": REQUIRED,
            "Headers": None,
            "Params": None,
        })
        res = self.client.get_object_to_file(
            key=req["Key"],
            filename=req["Filename"],
            headers=req["Headers"],
            params=req["Params"],
        )
        return {
            "Status": res.status,
            "Headers": dict([(i, res.headers[i]) for i in res.headers]),
            "RequestId": res.request_id,
            "Etag": res.etag,
            "ContentLength": res.content_length,
            "LastModified": res.last_modified,
            "VersionId": res.versionid,
            "DeleteMarker": res.delete_marker,
        }

    def put_object_from_file(self, req):
        req = merge(req, {
            "Key": REQUIRED,
            "Filename": REQUIRED,
            "Headers": None,
        })
        res = self.client.put_object_from_file(
            key=req["Key"],
            filename=req["Filename"],
            headers=req["Headers"],
        )
        return {
            "Status": res.status,
            "Headers": dict([(i, res.headers[i]) for i in res.headers]),
            "RequestId": res.request_id,
            "Etag": res.etag,
            "VersionId": res.versionid,
            "DeleteMarker": res.delete_marker,
        }

    def sign_url(self, req):
        req = merge(req, {
            "Method": "GET",
            "Key": REQUIRED,
            "Expires": 1800,
            "Headers": None,
            "Params": None,
            "SlashSafe": False,
        })
        res = self.client.sign_url(
            method=req["Method"],
            key=req["Key"],
            expires=req["Expires"],
            headers=req["Headers"],
            params=req["Params"],
            slash_safe=req["SlashSafe"],
        )
        return {
            "DownloadURL": res
        }




#!/usr/bin/env python3


import json
import unittest
import mns.mns_client
from mns.account import Account
from mns.topic import TopicMessage

from .mns_driver import MNSDriver

host = "http://xx.mns.cn-shanghai.aliyuncs.com"
access_key_id = "xx"
access_key_secret = "xx"
topic = "test-mns-topic-shanghai"
queue = "test-mns-queue-shanghai"


class TestMNSTopic(unittest.TestCase):
    def test_mns_client_public_message(self):
        client = mns.mns_client.MNSClient(
            host=host,
            access_id=access_key_id,
            access_key=access_key_secret,
        )

        req = mns.mns_client.PublishMessageRequest(
            topic_name=topic,
            message_body="hello world 123",
            message_tag="",
            direct_mail=None,
            direct_sms=None,
        )
        res = mns.mns_client.PublishMessageResponse()
        client.publish_message(req, res)
        print({
            "MessageId": res.message_id,
            "MessageBodyMD5": res.message_body_md5,
        })

    def test_mns_client_receive_message(self):
        client = mns.mns_client.MNSClient(
            host=host,
            access_id=access_key_id,
            access_key=access_key_secret,
        )

        req = mns.mns_client.ReceiveMessageRequest(
            queue_name=queue,
            base64decode=False,
            wait_seconds=1,
        )
        res = mns.mns_client.ReceiveMessageResponse()
        client.receive_message(req, res)
        res = {
            "MessageId": res.message_id,
            "ReceiptHandle": res.receipt_handle,
            "MessageBodyMD5": res.message_body_md5,
            "MessageBody": res.message_body,
            "EnqueueTime": res.enqueue_time,
            "NextVisibleTime": res.next_visible_time,
            "FirstDequeueTime": res.first_dequeue_time,
            "DequeueCount": res.dequeue_count,
            "Priority": res.priority,
            "Body": json.loads(res.message_body),
        }
        print(json.dumps(res, indent=True))

    def test_mns_topic(self):
        account = Account(
            host=host,
            access_id=access_key_id,
            access_key=access_key_secret,
        )
        client = account.get_topic(topic)
        # res = topic.publish_message(TopicMessage(base64.b64encode("hello world".encode("utf-8")).decode("utf-8")))
        res = client.publish_message(TopicMessage('hello world'))
        print(res.message_id)
        print({
            "MessageId": res.message_id,
            "MessageBodyMD5": res.message_body_md5,
        })

    # 这种方式会自动对 message body 进行 base64 解码，奇奇怪怪的，调试好久没调通，建议直接使用 mns_client 底层接口即可
    def test_mns_queue(self):
        account = Account(
            host=host,
            access_id=access_key_id,
            access_key=access_key_secret,
        )
        client = account.get_queue(queue)
        res = client.receive_message(1)
        res = {
            "MessageId": res.message_id,
            "ReceiptHandle": res.receipt_handle,
            "MessageBodyMD5": res.message_body_md5,
            "MessageBody": res.message_body,
            "EnqueueTime": res.enqueue_time,
            "NextVisibleTime": res.next_visible_time,
            "FirstDequeueTime": res.first_dequeue_time,
            "DequeueCount": res.dequeue_count,
            "Priority": res.priority,
        }
        print(res["MessageBody"])


class TestMNSDriver(unittest.TestCase):
    def setUp(self) -> None:
        self.driver = MNSDriver({
            "AccessKeyId": access_key_id,
            "AccessKeySecret": access_key_secret,
            "Endpoint": host,
        })

    def test_publish_message(self):
        res = self.driver.do({
            "Action": "PublishMessage",
            "Topic": topic,
            "MessageBody": "hello world 123",
            "Body": {},
        })
        print(json.dumps(res, indent=True))

    def test_publish_message_json(self):
        res = self.driver.do({
            "Action": "PublishMessage",
            "Topic": topic,
            "MessageBody": "",
            "Body": {
                "key1": "val1",
                "key2": "val2",
            },
        })
        print(json.dumps(res, indent=True))

    def test_receive_message(self):
        res = self.driver.do({
            "Action": "ReceiveMessage",
            "Queue": queue,
        })
        print(json.dumps(res, indent=True))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3


import hmac
import hashlib
import base64
import urllib.parse
import requests
from datetime import datetime

from .hook import Hook
from ..result import TestResult
from ..util import merge, REQUIRED


class DingTalkHook(Hook):
    def __init__(self, args):
        super().__init__(args)
        args = merge(args, {
            "accessToken": REQUIRED,
            "secret": REQUIRED,
            "actionURL": "",
        })

        self.access_token = args["accessToken"]
        self.secret = args["secret"]
        self.action_url = args["actionURL"]

    def on_exit(self, res: TestResult):
        ts, sign = calculate_sign(self.secret)

        btns = []
        if self.action_url:
            btns.append({
                "title": "查看详情",
                "actionURL": self.action_url,
            })

        res = requests.post(
            url="https://oapi.dingtalk.com/robot/send",
            params={
                "access_token": self.access_token,
                "timestamp": ts,
                "sign": sign,
            },
            headers={
                "Content-Type": "application/json"
            },
            json={
                "msgtype": "actionCard",
                "actionCard": {
                    "title": "{res.name} {i18n.title.report}".format(res=res, i18n=self.i18n),
                    "text": self.render_body(res),
                    "btnOrientation": "0",
                    "btns": btns
                }
            }
        )

        print(res.status_code)
        print(res.text)

    def render_body(self, res: TestResult):
        status = {
            "pass": self.i18n.status.succ,
            "fail": self.i18n.status.fail,
            "skip": self.i18n.status.skip,
        }[res.status]

        return """
#### {res.name} {i18n.title.test} {status}

- **{i18n.summary.casePass}**: {res.case_pass}
- **{i18n.summary.caseFail}**: {res.case_fail}
- **{i18n.summary.caseSkip}**: {res.case_skip}
""".format(res=res, i18n=self.i18n, status=status)


def calculate_sign(secret):
    timestamp = str(round(datetime.now().timestamp() * 1000))
    secret_enc = secret.encode('utf-8')
    string_to_sign = '{}\n{}'.format(timestamp, secret)
    string_to_sign_enc = string_to_sign.encode('utf-8')
    hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
    sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
    return timestamp, sign



#!/usr/bin/env python3


import json
import unittest
from .shell_driver import *


# https://docs.python.org/3/library/subprocess.html
class TestSubprocess(unittest.TestCase):
    def test_bash(self):
        process = subprocess.run(["/bin/bash", "-c", "echo ${PATH}"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print(process.returncode, process.stdout, process.stderr)

    def test_python(self):
        process = subprocess.run(["python3", "-c", "print('hello world')"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print(process.returncode, process.stdout, process.stderr)

    def test_env(self):
        process = subprocess.run(["/bin/bash", "-c", "echo ${KEY1}"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env={
            "KEY1": "val1"
        })
        print(process.returncode, process.stdout, process.stderr)


class TestShellDriver(unittest.TestCase):
    def setUp(self) -> None:
        self.driver = ShellDriver(args={})

    def test_bash(self):
        res = self.driver.do(req={
            "command": "echo hello world"
        })
        print(json.dumps(res))
        self.assertDictEqual(res, {
            "exitCode": 0,
            "stdout": "hello world\n",
            "stderr": "",
        })


class TestShellDriverPython(unittest.TestCase):
    def setUp(self) -> None:
        self.driver = ShellDriver(args={
            "shebang": "python3",
            "args": ["-c"],
        })

    def test_python(self):
        res = self.driver.do(req={
            "command": """
a = "hello"
b = "world"
print("{} {}".format(a, b))
"""
        })
        print(json.dumps(res))
        self.assertDictEqual(res, {
            "exitCode": 0,
            "stdout": "hello world\n",
            "stderr": "",
        })


if __name__ == '__main__':
    unittest.main()



#!usr/bin/env python3


import unittest
import pathlib
import sys
import os

from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol

from .thrift_driver import ThriftDriver


class TestThrift(unittest.TestCase):
    def test_thrift(self):
        p = pathlib.Path("../ops/thrift/tutorial.thrift")
        x = "{}/gen-py".format(p.parent.absolute())

        sys.path.append("{}/gen-py".format(p.parent.absolute()))
        socket = TSocket.TSocket('localhost', 9090)
        transport = TTransport.TBufferedTransport(socket)
        protocol = TBinaryProtocol.TBinaryProtocol(transport)
        transport.open()

        service = __import__("tutorial.Calculator", fromlist=["tutorial"])
        ttypes = __import__("tutorial.ttypes", fromlist=["tutorial"])
        client = getattr(service, "Client")(protocol)

        res = getattr(client, "calculate")(1, getattr(ttypes, "Work")(**{
            "num1": 8,
            "num2": 4,
            "op": 3
        }))
        print(res)

        transport.close()


class TestThriftDriver(unittest.TestCase):
    def test_thrift(self):
        print(os.getcwd())
        d = ThriftDriver(args={
            "proto": "../ops/thrift/tutorial.thrift",
            "endpoint": "localhost:9090",
            "service": "Calculator",
        })

        res = d.do({
            "method": "calculate",
            "args": [
                1234,
                {
                    "type": "Work",
                    "args": {
                        "num1": 8,
                        "num2": 4,
                        "op": 3
                    }
                }
            ]
        })
        print(res)

        res = d.do({
            "method": "getStruct",
            "args": [1234],
        })
        print(res)


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import copy
import re
import json

from ..result import SubStepResult


def format_step_res(sub_step: SubStepResult, separator="#", pass_open="", pass_close="", fail_open="", fail_close="") -> str:
    if len(sub_step.assertions) == 1 and sub_step.assertions[0].node == "":
        if sub_step.assertions[0].is_pass:
            return "{} {} {}{}{}".format(
                json.dumps(sub_step.res, indent=2), separator, pass_open, sub_step.assertions[0].expect, pass_close
            )
        else:
            return "{} {} {}{}{}".format(
                json.dumps(sub_step.res, indent=2), separator, fail_open, sub_step.assertions[0].expect, fail_close
            )

    # 修改 res 返回值，将预期值标记后拼接在 value 后面
    res = copy.deepcopy(sub_step.res)
    for expect_result in sub_step.assertions:
        if expect_result.is_pass:
            append_val_to_key(res, expect_result.node, "<GREEN>{}<END>".format(expect_result.expect))
        else:
            append_val_to_key(res, expect_result.node, "<RED>{}<END>".format(expect_result.expect))

    res_lines = json.dumps(res, indent=2).split("\n")
    lines = []
    # 解析 res 中 value 的值，重新拼接成带颜色的结果值
    for line in res_lines:
        mr = re.match(r'(\s+".*?": )"(.*)<GREEN>(.*)<END>"(.*)', line)
        if mr:
            lines.append("{}{}{} {} {}{}{}".format(
                mr.groups()[0], json.loads('"{}"'.format(mr.groups()[1])), mr.groups()[3], separator, pass_open, mr.groups()[2], pass_close
            ))
            continue
        mr = re.match(r'(\s+".*?": )"(.*)<RED>(.*)<END>"(.*)', line)
        if mr:
            lines.append("{}{}{} {} {}{}{}".format(
                mr.groups()[0], json.loads('"{}"'.format(mr.groups()[1])), mr.groups()[3], separator, fail_open, mr.groups()[2], fail_close
            ))
            continue
        mr = re.match(r'(\s+)"(.*)<GREEN>(.*)<END>"(.*)', line)
        if mr:
            lines.append("{}{}{} {} {}{}{}".format(
                mr.groups()[0], json.loads('"{}"'.format(mr.groups()[1])), mr.groups()[3], separator, pass_open, mr.groups()[2], pass_close
            ))
            continue
        mr = re.match(r'(\s+)"(.*)<RED>(.*)<END>"(.*)', line)
        if mr:
            lines.append("{}{}{} {} {}{}{}".format(
                mr.groups()[0], json.loads('"{}"'.format(mr.groups()[1])), mr.groups()[3], separator, fail_open, mr.groups()[2], fail_close
            ))
            continue
        lines.append(line)
    return '\n'.join(lines)


def append_val_to_key(vals, node, val):
    keys = node.split(".")
    for k in keys[:-1]:
        if isinstance(vals, dict):
            vals = vals[k]
        else:
            vals = vals[int(k)]

    key = keys[-1]
    if isinstance(vals, dict):
        if key in vals:
            lines = json.dumps(vals[key], indent=2).split("\n")
            for i in range(1, len(lines)):
                lines[i] =  "  " * len(keys) + lines[i]
            vals[key] = "{}{}".format("\n".join(lines), val)
        else:
            vals[key] = "{}{}".format(json.dumps(None), val)
    else:
        if int(key) < len(vals):
            lines = json.dumps(vals[int(key)], indent=2).split("\n")
            for i in range(1, len(lines)):
                lines[i] = "  " * len(keys) + lines[i]
            vals[int(key)] = "{}{}".format("\n".join(lines), val)
        else:
            vals.append("{}{}".format(json.dumps(None), val))



#!/usr/bin/env python3


import durationpy
import requests
from ..util import merge, REQUIRED
from .driver import Driver


class HttpDriver(Driver):
    endpoint = None

    def __init__(self, args: dict):
        args = merge(args, {
            "endpoint": REQUIRED,
        })

        self.endpoint = args["endpoint"].rstrip("/")

    def name(self, req):
        return req["path"] if "path" in req else "/"

    def do(self, req: dict):
        req = merge(req, {
            "endpoint": self.endpoint,
            "method": "POST",
            "headers": {},
            "params": {},
            "data": None,
            "json": None,
            "path": "",
            "timeout": "1s",
            "allowRedirects": True,
        })

        res = requests.request(
            method=req["method"],
            url="{}{}".format(req["endpoint"], req["path"]),
            params=req["params"],
            data=req["data"],
            json=req["json"],
            headers=req["headers"],
            timeout=durationpy.from_str(req["timeout"]).total_seconds(),
            allow_redirects=req["allowRedirects"],
        )

        body = None
        try:
            body = res.json()
        except Exception as e:
            pass

        return {
            "status": res.status_code,
            "headers": dict(res.headers),
            "json": body,
            "text": res.text,
        }



#!/usr/bin/env python3


import smtplib
import datetime
from email.mime.text import MIMEText
from email.header import Header
from jinja2 import Environment, BaseLoader

from .hook import Hook
from ..util import merge, REQUIRED
from ..result import TestResult


_test_tpl = """
<!DOCTYPE html>
<html lang="zh-cmn-Hans">
<head>
    <title>{{ res.name }} {{ i18n.title.report }}</title>
    <meta charset="UTF-8">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3" crossorigin="anonymous">
</head>
<body>
    <div class="row justify-content-md-center mt-5" id={{ name }}><div class="col-md-10">
        {% if res.is_skip %}
        <div class="card border-warning bg-warning">
            <h5 class="card-header text-white bg-transparent border-0">{{ i18n.title.test }} {{ res.name }} {{ i18n.status.skip }}</h5>
        {% elif res.is_pass %}
        <div class="card border-success">
            <h5 class="card-header text-white bg-success">{{ i18n.title.test }} {{ res.name }} {{ i18n.status.succ }}</h5>
        {% else %}
        <div class="card border-danger">
            <h5 class="card-header text-white bg-danger">{{ i18n.title.test }} {{ res.name }} {{ i18n.status.fail }}</h5>
        {% endif %}
    
            {% if not res.is_skip %}
            <div class="card-body">
                <table class="table table-striped">
                    <thead>
                        <tr class="text-center">
                            <th>{{ i18n.summary.caseTotal }}</th>
                            <th>{{ i18n.summary.casePass }}</th>
                            <th>{{ i18n.summary.caseSkip }}</th>
                            <th>{{ i18n.summary.caseFail }}</th>
                            <th>{{ i18n.summary.stepPass }}</th>
                            <th>{{ i18n.summary.stepSkip }}</th>
                            <th>{{ i18n.summary.stepFail }}</th>
                            <th>{{ i18n.summary.assertionPass }}</th>
                            <th>{{ i18n.summary.assertionFail }}</th>
                            <th>{{ i18n.summary.elapse }}</th>
                        </tr>
                    </thead>
                    <tbody>
                        <tr class="text-center">
                            {% if res.case_pass %}
                            <td><span class="badge bg-primary rounded-pill">{{ res.case_pass + res.case_skip + res.case_fail }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.case_pass }}</span></td>
                            {% endif %}
    
                            {% if res.case_pass %}
                            <td><span class="badge bg-success rounded-pill">{{ res.case_pass }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.case_pass }}</span></td>
                            {% endif %}
    
                            {% if res.case_skip %}
                            <td><span class="badge bg-warning rounded-pill">{{ res.case_skip }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.case_skip }}</span></td>
                            {% endif %}
    
                            {% if res.case_fail %}
                            <td><span class="badge bg-danger rounded-pill">{{ res.case_fail }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.case_fail }}</span></td>
                            {% endif %}
    
                            {% if res.step_pass %}
                            <td><span class="badge bg-success rounded-pill">{{ res.step_pass }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.step_pass }}</span></td>
                            {% endif %}
    
                            {% if res.step_skip %}
                            <td><span class="badge bg-warning rounded-pill">{{ res.step_skip }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.step_skip }}</span></td>
                            {% endif %}
    
                            {% if res.step_fail %}
                            <td><span class="badge bg-danger rounded-pill">{{ res.step_fail }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.step_fail }}</span></td>
                            {% endif %}
    
                            {% if res.assertion_pass %}
                            <td><span class="badge bg-success rounded-pill">{{ res.assertion_pass }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.assertion_pass }}</span></td>
                            {% endif %}
    
                            {% if res.assertion_fail %}
                            <td><span class="badge bg-danger rounded-pill">{{ res.assertion_fail }}</span></td>
                            {% else %}
                            <td><span class="badge bg-secondary rounded-pill">{{ res.assertion_fail }}</span></td>
                            {% endif %}
    
                            <td>{{ format_timedelta(res.elapse) }}</td>
                        </tr>
                    </tbody>
                </table>
            </div>
            {% endif %}
        </div>
    </div></div>
</body>
"""


class EmailHook(Hook):
    def __init__(self, args):
        super().__init__(args)
        args = merge(args, {
            "endpoint": REQUIRED,
            "username": REQUIRED,
            "password": REQUIRED,
            "receiver": REQUIRED,
            "disableOnPass": False,
        })

        host, port = args["endpoint"].split(":")
        self.host = host
        self.port = port
        self.password = args["password"]
        self.username = args["username"]
        self.receiver = args["receiver"]
        self.disable_on_pass = args["disableOnPass"]

    def on_exit(self, res: TestResult):
        if self.disable_on_pass and res.is_pass:
            return

        body = self.render(res)

        message = MIMEText(body, 'html', 'utf-8')
        message['From'] = Header(self.username, 'utf-8')
        message['To'] = Header(self.receiver, 'utf-8')
        message['Subject'] = Header("{} {}".format(res.name, self.i18n.title.report), 'utf-8')

        smtp = smtplib.SMTP_SSL(self.host, self.port)
        smtp.login(self.username, self.password)
        smtp.sendmail(self.username, self.receiver.split(";"), message.as_string())

    def render(self, res):
        env = Environment(loader=BaseLoader())
        env.globals.update(i18n=self.i18n)
        env.globals.update(format_timedelta=EmailHook.format_timedelta)
        tpl = env.from_string(_test_tpl)
        return tpl.render(res=res)

    @staticmethod
    def format_timedelta(t: datetime.timedelta):
        return "{:.3f}s".format(t.total_seconds())



#!/usr/bin/env python3


from .driver import Driver
from .shell_driver import ShellDriver
from .mysql_driver import MysqlDriver
from .redis_driver import RedisDriver
from .mongo_driver import MongoDriver
from .http_driver import HttpDriver
from .grpc_driver import GRPCDriver
from .thrift_driver import ThriftDriver
from .elasticsearch_driver import ElasticSearchDriver
from .pop_driver import POPDriver
from .ots_driver import OTSDriver
from .mns_driver import MNSDriver
from .oss_driver import OSSDriver
from .sls_driver import SLSDriver


driver_map = {
    "http": HttpDriver,
    "grpc": GRPCDriver,
    "thrift": ThriftDriver,
    "redis": RedisDriver,
    "shell": ShellDriver,
    "mysql": MysqlDriver,
    "mongo": MongoDriver,
    "elasticsearch": ElasticSearchDriver,
    "pop": POPDriver,
    "ots": OTSDriver,
    "mns": MNSDriver,
    "oss": OSSDriver,
    "sls": SLSDriver,
}


__all__ = [
    "Driver",
    "HttpDriver",
    "GRPCDriver",
    "ThriftDriver",
    "ShellDriver",
    "MysqlDriver",
    "RedisDriver",
    "MongoDriver",
    "MNSDriver",
    "POPDriver",
    "OTSDriver",
    "OSSDriver",
    "SLSDriver",
    "driver_map",
]



#!/usr/bin/env python3


import json
import yaml
import subprocess
import traceback

from ..util import merge, REQUIRED
from .driver import Driver


class ShellDriver(Driver):
    shebang: str
    args: list[str]
    envs: dict[str, str]

    def __init__(self, args):
        args = merge(args, {
            "shebang": "/bin/bash",
            "args": ["-c"],
            "envs": dict[str, str](),
        })
        self.shebang = args["shebang"]
        self.args = args["args"]
        self.envs = args["envs"]

    def name(self, req):
        return req["command"].split(" ")[0]

    def do(self, req):
        req = merge(req, {
            "command": REQUIRED,
            "envs": dict[str, str](),
            "decoder": "text",
        })

        process = subprocess.run(
            [self.shebang, *self.args, req["command"]],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            env=self.envs | req["envs"],
        )

        res = {
            "exitCode": process.returncode,
            "stdout": process.stdout.decode("utf-8"),
            "stderr": process.stderr.decode("utf-8"),
        }

        try:
            if req["decoder"] == "json":
                res["json"] = json.loads(res["stdout"])
            if req["decoder"] == "yaml":
                res["json"] = yaml.safe_load(res["stdout"])
        except Exception as e:
            res["err"] = traceback.format_exc()

        return res




#!/usr/bin/env python3


import unittest
from elasticsearch import Elasticsearch
from datetime import datetime

from .elasticsearch_driver import ElasticSearchDriver


class TestElasticSearch(unittest.TestCase):
    def test_elasticsearch(self):
        es = Elasticsearch(
            ["http://localhost:9200"],
        )

        res = es.index(index="test-index", id="123", document={
            "author": "kimchy",
            "text": "Elasticsearch: cool. bonsai cool.",
            "timestamp": datetime.now(),
        })
        print(res)

        res = es.get(index="test-index", id="123")
        print(res)

        res = es.search(index="test-index", query={"match_all": {}})
        print(res)

        res = es.delete(index="test-index", id="123")
        print(res)


class TestElasticSearchDriver(unittest.TestCase):
    def test_elasticsearch(self):
        driver = ElasticSearchDriver({
            "endpoint": "http://localhost:9200",
        })

        res = driver.do({
            "cmd": "index",
            "index": "test-index",
            "id": "123",
            "document": {
                "author": "kimchy",
                "text": "Elasticsearch: cool. bonsai cool.",
                "timestamp": datetime.now(),
            }
        })
        print(res)

        res = driver.do({
            "cmd": "get",
            "index": "test-index",
            "id": "123",
        })
        print(res)

        res = driver.do({
            "cmd": "search",
            "index": "test-index",
            "query": {
                "match_all": {}
            }
        })
        print(res)


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3


import unittest
from jinja2 import Environment, BaseLoader


class TestJinja(unittest.TestCase):
    def test_jinja(self):
        tpl = """
        <!DOCTYPE html>
        <html lang="en">
        <head>
            <title>My Webpage</title>
        </head>
        <body>
            <ul id="navigation">
            {% for item in navigation %}
                <li><a href="{{ item.href }}">{{ item.caption }}</a></li>
            {% endfor %}
            </ul>

            <h1>My Webpage</h1>
            {{ a_variable }}

            {# a comment #}
        </body>
        </html>
        """

        template = Environment(loader=BaseLoader).from_string(tpl)
        data = template.render(navigation=[{
            "href": "href1",
            "caption": "caption1",
        }, {
            "href": "href2",
            "caption": "caption2",
        }], a_variable="hello world")
        print(data)


tpl = """key: {{ val.key }}
{% for sub in val.subs %}
{% print(render(sub)) %}
{% endfor %}
"""


class TestJinja2(unittest.TestCase):
    def setUp(self) -> None:
        env = Environment(loader=BaseLoader)
        env.globals.update(render=self.render)
        self.template = env.from_string(tpl)

    def render(self, val):
        return "\n".join(["  " + line for line in self.template.render(val=val).split("\n")])

    def test_recursive(self):
        res = self.template.render(val={
            "key": "val1",
            "subs": [{
                "key": "val2",
                "subs": [{
                    "key": "val3",
                }]
            }, {
                "key": "val4",
                "subs": [{
                    "key": "val5"
                }, {
                    "key": "val6"
                }]
            }]
        })
        print(res)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3


import pathlib
import sys
import subprocess
import importlib
import grpc

from ..util import merge, REQUIRED
from .driver import Driver
from google.protobuf.json_format import MessageToJson, MessageToDict


class GRPCDriver(Driver):
    def __init__(self, args):
        args = merge(args, {
            "proto": REQUIRED,
            "endpoint": REQUIRED,
            "service": REQUIRED
        })
        p = pathlib.Path(args["proto"])
        self.proto_path = p
        self.generate_proto()
        sys.path.append(str(p.parent.absolute()))
        self.pb2 = importlib.import_module(p.stem + "_pb2")
        self.pb2_grpc = importlib.import_module(p.stem + "_pb2_grpc")
        channel = grpc.insecure_channel(args["endpoint"])
        self.stub = getattr(self.pb2_grpc, args["service"])(channel)

    def generate_proto(self):
        command = "python3 -m grpc_tools.protoc " \
                  "-I{directory} --python_out={directory} " \
                  "--grpc_python_out={directory} {source}".format(
                    directory=str(self.proto_path.parent), source=str(self.proto_path))
        subprocess.run(
            command.split(),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

    def name(self, req):
        return req["method"]

    def do(self, req):
        req = merge(req, {
            "method": REQUIRED,
            "type": REQUIRED,
            "args": {},
        })

        res = getattr(self.stub, req["method"])(getattr(self.pb2, req["type"])(**req["args"]))

        return MessageToDict(res)
